import { ComponentConstructorInfo, ComponentDrawInfo, GridConfig } from '../types';
import Component from '../common/Component';
import AxisMilestone from '../Axis/AxisMilestone';
import AxisScale from '../Axis/AxisScale';
import AxisBody from '../Axis/AxisBody';
import { SN } from '../common/definitions';
/**
 * @property {number[]} scales - the position of scales. Every item should <= 1.
 * @property {Object[]} milestones - the config of milestones.
 * @property {number} milestones[].position. - the position of milestone. Every item should <= 1.
 * @property {string} milestones[].text. - the showed text of milestone.
 * @property {number} length - the axis length.
 * */
interface DrawInfo extends ComponentDrawInfo {
    scales: number[];
    milestones: {
        position: number;
        content: AxisMilestone['drawInfo']['content'];
    }[];
    length: number;
}
/**
 * @property {GridConfig} grid - the whole config of GRID.
 * */
export interface ConstructInfo extends ComponentConstructorInfo {
    grid: GridConfig;
}
/**
 * The whole Axis. Contain some AxisMilestone, some AxisScale and a AxisBody.
 * The Manage-Component. All of draw is resolved by child component.
 * */
export default abstract class Axis extends Component {
    constructor(props: ConstructInfo);
    grid: GridConfig;
    name: SN;
    drawInfo: DrawInfo;
    milestones: AxisMilestone[];
    scales: AxisScale[];
    body: AxisBody;
    abstract milestoneConstructor: typeof AxisMilestone;
    abstract scaleConstructor: typeof AxisScale;
    abstract bodyConstructor: typeof AxisBody;
    createBox(): void;
    apply(): Promise<import("../common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD>;
    draw(): import("../common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD;
    destroy(): import("../common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD;
    hide(): import("../common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD;
    initBody(): void;
    initScales(): void;
    initMilestones(): void;
    static is(comp: Component): comp is Axis;
}
export {};
