import Component from '../common/Component';
import { ComponentConstructorInfo, ComponentDrawInfo, Coordinate, GridConfig } from '../types';
import EventMark from './EventMark';
import EventBody from './EventBody';
import EventAxis from './EventAxis';
import { SN } from '../common/definitions';
import AxisBody from '../Axis/AxisBody';
/**
 * @property {Coordinate} target - a coordinate the Event need to approach.
 * @property {Coordinate} offset - a coordinate offset with the this.target.
 *
 * @property {Date} date - a data of event.
 * @property {string} title - the title of event.
 * @property {[string]} description - the description of event.
 * @property {boolean} folded - indicate the EventBody was in fold-mode or not.
 * @property {[string]} foldPlaceholder - a text to show in EventBody when it was in fold-mode.
 *
 * @property {[number]} axisLength - the length of EventAxis.
 * @property {[number]} axisOffset - the offset X with Axis in EventAxis.
 * @property {[Date]} endDate - the date of event end.
 * @property {[string]} endText - the description about event ended.
 * */
interface DrawInfo extends ComponentDrawInfo {
    target: Coordinate;
    offset: Coordinate;
    date: string;
    title: string;
    description?: string;
    folded: boolean;
    foldPlaceholder?: string;
    endDate?: string;
    endText?: string;
    axisLength?: number;
    axisOffset?: number;
    axisBodyDrawInfo: Readonly<AxisBody['drawInfo']>;
}
/**
 * @property {GridConfig} grid - the whole config of GRID.
 * */
export interface ConstructInfo extends ComponentConstructorInfo {
    grid: GridConfig;
}
/**
 * The whole Event. Contain an EventBody and an EventMark, maybe has a EventAxis too.
 * The Manage-Component. All of draw is resolved by child component.
 * */
export default abstract class Event extends Component {
    constructor(props: ConstructInfo);
    grid: GridConfig;
    name: SN;
    drawInfo: DrawInfo;
    mark: EventMark;
    body: EventBody;
    axis: EventAxis | null;
    abstract bodyConstructor: typeof EventBody;
    abstract axisConstructor: typeof EventAxis;
    abstract markConstructor: typeof EventMark;
    createBox(): void;
    apply(): Promise<import("../common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD>;
    draw(): import("../common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD;
    hide(): import("../common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD;
    destroy(): import("../common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD;
    initMark(): void;
    initBody(): void;
    initAxis(): void;
    static is(comp: Component): comp is Event;
}
export {};
