"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var definitions_1 = require("../common/definitions");
var moveto = require("moveto");
var Axis_1 = __importDefault(require("../Axis"));
var AxisBody_1 = __importDefault(require("../Axis/AxisBody"));
var Timeline_1 = __importDefault(require("../Timeline"));
/**
 * Supported all of Breakpoint.
 * @property {string} PushScalesAndMilestones
 * When recompute the position of Scales and Milestones
 * @property {string} FixEventBody2AxisMilestone
 * When fix conflict between EventBody and AxisMilestone.
 * @property {string} FixEventAxis2EventAxis
 * @property {string} FixEventBody2EventBodyMover
 * @property {string} FixEventBody2EventBodyFloater
 * @property {string} DrawFrom - when draw a timeline by data imported.
 * @property {string} Debug
 * Set a Breakpoint for debug.
 * When debug-mode enabled, short-night will blocking at Breakpoint.Debug automatically.
 * */
var Breakpoint;
(function (Breakpoint) {
    Breakpoint["PushScalesAndMilestones"] = "PushScalesAndMilestones";
    Breakpoint["FixEventBody2AxisMilestone"] = "FixEventBody2AxisMilestone";
    Breakpoint["FixEventAxis2EventAxis"] = "FixEventAxis2EventAxis";
    Breakpoint["FixEventBody2EventBodyMover"] = "FixEventBody2EventBodyMover";
    Breakpoint["FixEventBody2EventBodyFloater"] = "FixEventBody2EventBodyFloater";
    Breakpoint["DrawFrom"] = "DrawFrom";
    Breakpoint["Debug"] = "Debug";
})(Breakpoint = exports.Breakpoint || (exports.Breakpoint = {}));
/**
 * 1. Support that set a Breakpoint .
 * 2. Support that play animation base on Breakpoints.
 * */
var BreakpointAnimation = /** @class */ (function () {
    function BreakpointAnimation(ext, _a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.breakpoints, breakpoints = _c === void 0 ? [] : _c, _d = _b.playAnimation, playAnimation = _d === void 0 ? false : _d, _e = _b.autoScroll, autoScroll = _e === void 0 ? false : _e, _f = _b.timeoutCounter, timeoutCounter = _f === void 0 ? function (point, config) {
            if (point === Breakpoint.DrawFrom)
                return 100;
            return config.forward ? 200 : 300;
        } : _f, _g = _b.scrollDuration, scrollDuration = _g === void 0 ? 50 : _g, _h = _b.scrollContainer, scrollContainer = _h === void 0 ? window : _h;
        this.ext = ext;
        this.breakpoints = breakpoints;
        this.playAnimation = playAnimation;
        this.autoScroll = autoScroll;
        this.timeoutCounter = timeoutCounter;
        this.scrollDuration = scrollDuration;
        this.scrollContainer = scrollContainer;
        if (this.playAnimation && this.breakpoints.length === 0) {
            this.breakpoints = BreakpointAnimation.defaultAnimationBreakpoints;
        }
        if (definitions_1.DEBUG) {
            this.breakpoints.push(Breakpoint.Debug);
            window.next = this.next.bind(this);
            window.abort = function () { delete _this.stepIn; };
        }
    }
    /**
     * Scroll to top when end of play animation.
     * */
    BreakpointAnimation.prototype.onDraw = function (comp) {
        if (Timeline_1.default.is(comp) && this.playAnimation) {
            this.moveTo.move(this.ext.components[definitions_1.SN.Timeline][0].canvas);
        }
    };
    /**
     * Blocking at a breakpoint util next called
     * */
    BreakpointAnimation.prototype.block = function (point, config) {
        if (config === void 0) { config = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var onBlock, onNext, _a, components, scrollTarget, scrollDistance;
            var _this = this;
            return __generator(this, function (_b) {
                onBlock = config.onBlock, onNext = config.onNext, _a = config.components, components = _a === void 0 ? [] : _a;
                if (!this.breakpoints.includes(point))
                    return [2 /*return*/];
                console.log("Blocking at " + point + ".");
                if (this.playAnimation) {
                    if (this.autoScroll) {
                        scrollTarget = this.countScrollTarget(config);
                        if (scrollTarget) {
                            scrollDistance = scrollTarget - document.documentElement.scrollTop;
                            if (Math.abs(scrollDistance) > 10) {
                                this.moveTo.move(scrollTarget - document.documentElement.scrollTop);
                            }
                        }
                    }
                    setTimeout(function () { return _this.next(); }, this.timeoutCounter(point, config));
                }
                return [2 /*return*/, (function () { return __awaiter(_this, void 0, void 0, function () {
                        var _this = this;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    if (!onBlock) return [3 /*break*/, 2];
                                    return [4 /*yield*/, onBlock()];
                                case 1:
                                    _a.sent();
                                    _a.label = 2;
                                case 2:
                                    components.forEach(function (comp) { return comp.draw(); });
                                    return [4 /*yield*/, new Promise(function (resolve) {
                                            _this.stepIn = function () { return __awaiter(_this, void 0, void 0, function () {
                                                return __generator(this, function (_a) {
                                                    switch (_a.label) {
                                                        case 0:
                                                            if (!onNext) return [3 /*break*/, 2];
                                                            return [4 /*yield*/, onNext()];
                                                        case 1:
                                                            _a.sent();
                                                            _a.label = 2;
                                                        case 2:
                                                            components.forEach(function (comp) { return comp.hide(); });
                                                            resolve();
                                                            return [2 /*return*/];
                                                    }
                                                });
                                            }); };
                                        })];
                                case 3:
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    }); })()];
            });
        });
    };
    /**
     * Step into next breakpoint.
     * You can call window.next() to do same thing when DEBUG is true.
     * */
    BreakpointAnimation.prototype.next = function () {
        if (!this.stepIn) {
            throw new Error('Call next when not blocking.');
        }
        this.stepIn();
        delete this.stepIn;
    };
    Object.defineProperty(BreakpointAnimation.prototype, "moveTo", {
        /** Get a instance of moveto */
        get: function () {
            return new moveto({
                duration: this.scrollDuration,
                container: this.scrollContainer,
            });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Count a number that be used scroll to focus when play animation.
     * */
    BreakpointAnimation.prototype.countScrollTarget = function (config) {
        var components = config.components, protagonist = config.protagonist;
        var topPadding = 100;
        if (protagonist)
            return protagonist.drawInfo.box.y - topPadding;
        if (components && components.length) {
            var getTopped = function (compList) { return compList.find(function (comp1) { return compList.every(function (comp2) { return comp1.drawInfo.box.y >= comp2.drawInfo.box.y; }); }); };
            var componentListWithoutAxis = components.filter(function (comp) { return !Axis_1.default.is(comp) && !AxisBody_1.default.is(comp); });
            return getTopped(componentListWithoutAxis).drawInfo.box.x - topPadding;
        }
        return null;
    };
    /**
     * @see BreakpointAnimationConfig.playAnimation
     * */
    BreakpointAnimation.defaultAnimationBreakpoints = [
        Breakpoint.FixEventBody2AxisMilestone,
        Breakpoint.FixEventAxis2EventAxis,
        Breakpoint.FixEventBody2EventBodyMover,
        Breakpoint.FixEventBody2EventBodyFloater,
        Breakpoint.DrawFrom,
    ];
    return BreakpointAnimation;
}());
exports.default = BreakpointAnimation;
