import { Extension, ExtensionManager } from '.';
import Component from '../common/Component';
import Axis from '../Axis';
import Timeline from '../Timeline';
/**
 * Set a point that some components need move if they upper has this point.
 * @property {number} critical - a critical point to judge a component need move or not.
 * @property {number} additional - what number a component need move for this config.
 * @property {Component} component - which component made this config.
 * */
interface PushConfig {
    critical: number;
    additional: number;
    component: Component;
}
/**
 * Milestone shouldn't occupy the space of Axis.
 * So we move Event, Scale and Milestone if they upper has any Milestones.
 * */
export default class PositionCounter implements Partial<Extension> {
    ext: ExtensionManager;
    constructor(ext: ExtensionManager);
    onApply(comp: Component): Promise<void>;
    /**
     * When a component destroyed, remove a config made by it.
     * */
    onDestroy(comp: Component): void;
    adjustAxis(axis: Axis): Promise<void>;
    adjustEvent(timeline: Timeline): Promise<void>;
    protected pushConfigs: PushConfig[];
    protected addPushConfig(config: PushConfig): void;
    protected countCritical(num: number, comp?: Component): number;
    protected updatePushConfigs(axis: Axis): Promise<void>;
}
export {};
