import Component from '../common/Component';
import { SN } from '../common/definitions';
import Timeline from '../Timeline';
import Axis from '../Axis';
import AxisBody from '../Axis/AxisBody';
import AxisScale from '../Axis/AxisScale';
import AxisMilestone from '../Axis/AxisMilestone';
import Event from '../Event';
import EventBody from '../Event/EventBody';
import EventMark from '../Event/EventMark';
import EventAxis from '../Event/EventAxis';
import BreakpointAnimation, { BreakpointAnimationConfig } from './BreakpointAnimation';
import BoxElementGenerator from './BoxElementGenerator';
import IdGenerator from './IdGenerator';
import PositionCounter from './PositionCounter';
import ConflictFixer from './ConflictFixer';
export { BoxElementGenerator, IdGenerator, PositionCounter, BreakpointAnimation, ConflictFixer, };
/**
 * @property {string} Debug - enable the extension both normal and disableAllButDebug
 * @property {string} DebugOnly - enable the extension only the debug is true
 * */
export declare enum ExtensionType {
    Debug = "debug",
    DebugOnly = "debug-only"
}
export interface Extension {
    /**
     * Specify the purpose of extension.
     * */
    type?: ExtensionType;
    onConstruct(comp: Component): void;
    onApply(comp: Component): Promise<void>;
    onDraw(comp: Component): void;
    onHide(comp: Component): void;
    onDestroy(comp: Component): void;
}
export interface ExtraData {
}
/**
 * Manage ExtensionManager#components.
 * */
export declare class Base implements Partial<Extension> {
    ext: ExtensionManager;
    constructor(ext: ExtensionManager);
    onConstruct(comp: Component): void;
    onDestroy(comp: Component): void;
}
export declare class ExtensionManager implements Extension {
    constructor({ breakpointAnimation, disableAll, disableAllButDebug }?: {
        disableAll?: boolean;
        disableAllButDebug?: boolean;
        breakpointAnimation?: Partial<BreakpointAnimationConfig>;
    });
    /**
     * Disable all of extensions but enable the extensions supported debug.
     * */
    disableAllButDebug: boolean;
    /**
     * Disable all of extensions.
     * */
    disableAll: boolean;
    /**
     * A shared object for every Component.
     * For extends, re-declare the ExtData.
     * */
    extraData: ExtraData;
    breakpoint: BreakpointAnimation;
    components: {
        [SN.Timeline]: Timeline[];
        [SN.Axis]: Axis[];
        [SN.AxisBody]: AxisBody[];
        [SN.AxisScale]: AxisScale[];
        [SN.AxisMilestone]: AxisMilestone[];
        [SN.Event]: Event[];
        [SN.EventBody]: EventBody[];
        [SN.EventMark]: EventMark[];
        [SN.EventAxis]: EventAxis[];
    };
    extensions: Partial<Extension>[];
    getParent(comp: EventAxis | EventBody | EventMark): Event;
    getParent(comp: AxisBody | AxisMilestone | AxisScale): Axis;
    getParent(comp: Event | Axis): Timeline;
    onConstruct: (comp: Component) => Promise<void>;
    onApply: (comp: Component) => Promise<void>;
    onHide: (comp: Component) => Promise<void>;
    onDraw: (comp: Component) => Promise<void>;
    onDestroy: (comp: Component) => Promise<void>;
}
